
;*************************
;*  GBC Remote Control   *
;*************************
; By Jeff Frohwein
; Last edit 2-Jan-99

; High RAM Usage
SpriteDMA       EQU     $ff80
StackPtr        EQU     $fffe
BUFSIZE         EQU     2048-1

; Carriers (kHz) - 32,36,37.916,39.2,40,56.875

; bit width (uS) = 400,520,550,600,840,888
; old panasonic & technics = 420 & 455 uS
; Denon 275pulse/1900space,275pulse/775space
; finlux 500+530
; jvc 35.5khz,500/600uS,500/1800uS
; mitsubishi 300/800,300/1950uS

;sample rate (3.8147uS)

        INCLUDE "hardware.inc"
        INCLUDE "ibmpc1.inc"

RGBSet:         MACRO
        DW      ((\3 >> 3) << 10) + ((\2 >> 3) << 5) + (\1 >> 3)
        ENDM

;Version 1.2 or later of 'hardware.inc' is required.
        rev_Check_hardware_inc 1.5
;Version 1.0 or later of 'load1.inc' is required.
;        rev_Check_load1_inc 1.0

        SECTION "Low Ram",BSS

; * SpriteTable *
SpriteTab       DS      40*4
IRRecBuf        DS      BUFSIZE+1
VBlankDone      DS      1
HeaderFlag      DS      1
temp            DS      2
count           DS      1
AvgVal          DS      2

;pressed         DS      1
;Counter         DS      1

        SECTION "Org $0",HOME
        ret


        SECTION "Org $8",HOME[$8]
        ret

        SECTION "Org $40",HOME[$40]
        reti

        SECTION "Org $50",HOME[$50]
        reti

        SECTION "Org $58",HOME[$58]
        reti

; DMA copy sprite data to sprite ram

VBlankFirst:
        ld      a,$c0 ;SpriteTable/256
        ldh     [rDMA],a
        ld      a,$28
VBlnkW: dec     a
        jr      nz,VBlnkW
        ret
VBlankLast:


VBlankIntr:
        push    af
        push    bc
        push    de
        push    hl

        call    SpriteDMA

; Put a state machine for updating 1 of 4
; sets of video data during this VBlank Interrupt.

        ld      a,1
        ld      [VBlankDone],a

        pop     hl
        pop     de
        pop     bc
        pop     af
        reti


        DB      "Date 981017"          ; Date stamping info for
                                       ; reference use only
        SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

        nop
        jp      ColdBoot

        NINTENDO_LOGO                   ; Nintendo graphic logo

;Rom Header Info

;    0123456789ABCDEF
 DB "IR DEMO #1",0,0,0,0,0    ; Cart name   15 bytes


 ; *********** NOTE ************
 ; THE FOLLOWING BYTE MUST BE $80
 ; OR THE IR PORT WILL NOT WORK!!


 DB $80                       ; $00 = GB/SGB, $80 = CGB
 DB $0,$0
 DB 0                         ; SGB = 3
 DB CART_ROM_MBC1_RAM_BAT   ;$19               ; Cart type
 DB CART_ROM_256K             ; ROM Size (in bits)
 DB CART_RAM_NONE             ; RAM Size (in bits)
 DB 1                         ; Country code
 DB $33                       ; Maker ID (Old)
 DB 0                         ; Version     =0
 DB 0                         ; Complement check (important)
 DW 0                         ; Checksum (not important)

; Macros

        INCLUDE "ibmpc1.inc"

; Library routines

        INCLUDE "memory1.asm"   ; memory set/copy
        INCLUDE "keypad.asm"    ; keypad

        SECTION "Main Code",HOME

        INCLUDE "magnavx1.inc"
ColdBoot:
WarmBoot:
        di
        ld      sp,StackPtr                ; Initialize stack pointer

; *** Initialize screen stuff ***

        call    Initialize              ; Init screen crap

; *** Display screen text message ***


        call    DisplayScreenMsgs

; *** Screen on ***

        call    ScreenOn

        ei

;        call    Learn

; *** Start Main Loop ***

MainLoop:
        call    pad_Read

        push    af
        call    DrawButtons
        pop     af

        push    af
        and     $0f
        cp      $0f
        call    z,Learn
        pop     af

        bit     PADB_UP,a
        jr      z,NotUp
        call    ChannelUp
        jr      EndOfMain
NotUp:
        bit     PADB_DOWN,a
        jr      z,NotDown
        call    ChannelDown
        jr      EndOfMain
NotDown:
        bit     PADB_LEFT,a
        jr      z,NotLeft
        call    VolumeDown
        jr      EndOfMain
NotLeft:
        bit     PADB_RIGHT,a
        jr      z,NotRight
        call    VolumeUp
        jr      EndOfMain
NotRight:
        bit     PADB_SELECT,a
        jr      z,NotSelect
        call    VolumeMute
        jr      EndOfMain
NotSelect:
        bit     PADB_START,a
        jr      z,MainLoop
        call    PowerOnOff

EndOfMain:
        call    IRDelayOff
        jr      MainLoop


Learn:
        call    ClearScreen

; Print msg for user to press an IR button

        call    PressIRButton

; Obtain data from IR input

        call    GetAllRaw

        call    DetectHeader
        call    CalcCarrierFreq
        push    bc

; Print msg for user to press an IR button

        call    PressIRButtonAgain

; Obtain data from IR input

        call    GetAllRaw

        call    DetectHeader
        call    CalcCarrierFreq
        pop     af

        cp      b               ; Do the frequencies match?
        jp      z,FreqOk        ; yes

        call    BadData
        jr      Learn

FreqOk:
; Calculate statistics & display

        call    Stats

TScrn:
        call    ClearScreen

; Print msg for user to press an IR button

        call    PressIRButton

; Calculate T value

        call    CalcT
        ld      h,d
        ld      l,e
        add     hl,bc           ; hl = pulse length + space length

        push    hl

; Print msg for user to press an IR button

        call    PressIRButtonAgain

; Calculate T value

        call    CalcT
        ld      h,d
        ld      l,e

        pop     de

        push    hl              ; save pulse length
        push    bc              ; save space length

        add     hl,bc           ; hl = pulse length + space length

        ld      b,3
        call    HLDEClose       ; Are DE & HL close in value?
        jr      c,VeryClose     ; yes

        pop     hl
        pop     hl
        call    BadData
        jr      TScrn

VeryClose:

        call    ClearScreen

        pop     de              ; get space length
        pop     hl              ; get pulse length

        ld      b,20
        call    HLDEClose       ; Are DE & HL close in value?
        ld      bc,EncodeRC5
        jr      c,DispEncode    ; yes, it's RC5 encoding

        ld      bc,EncodePulse
        ld      a,h
        cp      d
        jr      nz,VClose2
        ld      a,l
        cp      e
VClose2:
        jr      nc,DispEncode

        ld      bc,EncodeSpace

; Display encoding format

DispEncode:
        push    de              ; save space length
        push    hl              ; save pulse length

        push    bc

        ld      hl,EncodeMsg
        ld      de,_SCRN0+0+(32*3)
        ld      bc,20
        call    mem_CopyVRAM

        pop     hl
        ld      de,_SCRN0+0+(32*4)
        ld      bc,20
        call    mem_CopyVRAM

 if 0
        pop     de              ; get pulse length

        ld      hl,_SCRN0+0+(32*16)
        ld      a,d
        call    DisplayHexByte
        ld      a,e
        call    DisplayHexByte

        ld      b,d
        ld      c,e

        pop     de              ; get space length
        ld      hl,_SCRN0+6+(32*16)
        ld      a,d
        call    DisplayHexByte
        ld      a,e
        call    DisplayHexByte

        push    de
        push    bc
 endc

; Display T length

        ld      hl,TMsg
        ld      de,_SCRN0+0+(32*7)
        ld      bc,20
        call    mem_CopyVRAM

;klop2:  jr      klop2


        pop     hl              ; get pulse length
        pop     de              ; get space length

        call    SortHLDE        ; Sort hl & de. Put smaller number in de.

        add     hl,de           ; hl = pulse length + space length


; HL is some multiple of T.
; (i.e. 2T,3T,4T,5T,etc...)
; Reduce HL to value of T.

        ld      c,2
ReduceHL:
        push    hl
        push    de

        push    hl
        pop     de

; 16 bit division
; DE = DE / BC, BC = remainder

        ld      b,0
        push    bc
        call    div1616
        pop     bc
        inc     c

        ld      h,d
        ld      l,e

        pop     de
        ld      b,40
        call    HLDEClose       ; Are DE & HL close in value?
        ld      a,l
        ld      [AvgVal],a
        ld      a,h
        ld      [AvgVal+1],a

        pop     hl

        jr      nc,ReduceHL     ; no

        ld      a,[AvgVal]
        ld      l,a
        ld      a,[AvgVal+1]
        ld      h,a

        ld      de,38
        call    mul_DE_HL_BCDEu ;de = 3.8147 x 10 us * AvgVal

        ld      h,d
        ld      l,e

        ld      de,_SCRN0+4+(32*8)
        call    bta

        dec     de
        ld      h,d
        ld      l,e

        ld      b,"u"
        call    WriteBtoVRAM
        ld      b,"S"
        call    WriteBtoVRAM

;        ld      a,d
;        call    DisplayHexByte
;        ld      a,e
;        call    DisplayHexByte

; Display press IR button again msg

        call    PressIRButtonBottom

        jp      WarmBoot

;klop:   jr      klop

;                12345678901234567890
TMsg:   db      " T (basic timing) = "

EncodeMsg:
        db      "Encoding format =   "
EncodeRC5:
        db      " RC5/Biphase Encode "
EncodePulse:
        db      " Pulse Length Encode"
EncodeSpace:
        db      " RECS80/Space Encode"
; Plot data obtained from IR input

        ld      hl,$9800+(32*0)
        ld      de,IRRecBuf
        ld      b,18
Loop1:
        push    bc
        ld      b,20
Loop2:
        ld      a,[de]
        inc     de
        rrca
        and     1
        add     $30
        ld      c,a
        di
.wait:
        ldh     a,[$41]
        and     2
        jr      nz,.wait

        ld      a,c
        ld      [hl+],a
        ei

        dec     b
        jr      nz,Loop2

        ld      bc,32-20
        add     hl,bc

        pop     bc
        dec     b
        jr      nz,Loop1

StopNow:
        jr      StopNow

        jp      MainLoop

; Write B to VRAM at HL

WriteBtoVRAM:
.loop:  ld      a,[$ff41]
        and     2
        jr      nz,.loop

        ld      [hl],b
        inc     hl
        ret

; Sort hl & de. Put smaller number in de.

SortHLDE:
        ld      a,h
        cp      d
        jr      nz,Sort1
        ld      a,l
        cp      e
Sort1:
        jr      nc,Sort2
        push    hl
        ld      h,d
        ld      l,e
        pop     de
Sort2:
        ret

; Frequency compare is bad.
; Tell user to cycle power.

BadData:
        ld      hl,pmsg1
        ld      de,_SCRN0+0+(32*13)
        ld      bc,20
        call    mem_CopyVRAM

        ld      hl,pmsg2
        ld      de,_SCRN0+0+(32*14)
        ld      bc,20
        call    mem_CopyVRAM

        ld      hl,pmsg3
        ld      de,_SCRN0+0+(32*15)
        ld      bc,20
        call    mem_CopyVRAM

.loop:  call    pad_Read
        or      a
        jr      z,.loop

        ret

;        jp      Learn

;                12345678901234567890
pmsg1:  db      "  IR data error.    "
pmsg2:  db      "  Please press A    "
pmsg3:  db      "  or B to retry.    "

;pmsg1:  db      " Freq compare error."
;pmsg2:  db      " Power down, power  "
;pmsg3:  db      " up & try again.    "

HdrNo:  db      "Header = No "
HdrYes: db      "Header = Yes"


; Check to see if DE & HL are close
; in value to each other (+/- 2)

; Exit: Set Carry flag if close

HLDEClose:
        push    de
        push    hl

        ld      a,d
        cpl
        ld      d,a
        ld      a,e
        cpl
        ld      e,a
        inc     de

        add     hl,de

        bit     7,h             ; Is hl negative?
        jr      z,HLDE2         ; no

        ld      a,h
        cpl
        ld      h,a
        ld      a,l
        cpl
        ld      l,a
        inc     hl
HLDE2:

        ld      a,h
        or      a
        jr      nz,HLDE3        ; return if hl > 255. Carry is reset.

        ld      a,l
        cp      b               ; Set carry if delta is smaller than B
HLDE3:
        pop     hl
        pop     de
        ret

; Calculate & display statistics for captured data

Stats:
        push    bc
        call    ClearScreen
        pop     bc

        call    DisplayCarrierFreq

        ld      a,[HeaderFlag]
        or      a
        ld      hl,HdrNo
        jr      z,Stats2
        ld      hl,HdrYes
Stats2:
        ld      de,_SCRN0+4+(32*8)
        ld      bc,12
        call    mem_CopyVRAM

; Print msg for user to press an IR button

        call    PressIRButtonBottom

        ret

; Clear the screen

ClearScreen:
        ld      hl,$9800
        ld      bc,$400
CLS2:
        ld      a,[$ff41]
        and     2
        jr      nz,CLS2

        ld      [hl],32
        inc     hl

        dec     bc
        ld      a,b
        or      c
        jr      nz,CLS2

        ret

; Display press IR button msg

PressIRButton:
        ld      hl,lmsg1
        ld      de,_SCRN0+0+(32*3)
        ld      bc,20
        call    mem_CopyVRAM

        ld      hl,lmsg2
        ld      de,_SCRN0+0+(32*4)
        ld      bc,20
        call    mem_CopyVRAM

        ld      hl,lmsg3
        ld      de,_SCRN0+0+(32*5)
        ld      bc,20
        call    mem_CopyVRAM

        ret
;                12345678901234567890

lmsg1:  db      "Point remote at end "
lmsg2:  db      " of GBC and press   "
lmsg3:  db      " any button on it.  "

; Display press IR button again msg

PressIRButtonAgain:
        call    IRDelayOff
        call    IRDelayOff
        call    IRDelayOff
        call    IRDelayOff
        call    IRDelayOff
        call    IRDelayOff

        ld      hl,lmsg1
        ld      de,_SCRN0+0+(32*8)
        ld      bc,20
        call    mem_CopyVRAM

        ld      hl,lmsg2
        ld      de,_SCRN0+0+(32*9)
        ld      bc,20
        call    mem_CopyVRAM

        ld      hl,lmsg3
        ld      de,_SCRN0+0+(32*10)
        ld      bc,20
        call    mem_CopyVRAM

        ret

; Display press IR button again msg

PressIRButtonBottom:
;        call    IRDelayOff
;        call    IRDelayOff
;        call    IRDelayOff
;        call    IRDelayOff
;        call    IRDelayOff
;        call    IRDelayOff

        ld      hl,cmsg1
        ld      de,_SCRN0+0+(32*11)
        ld      bc,20
        call    mem_CopyVRAM

;        ld      hl,lmsg2
;        ld      de,_SCRN0+0+(32*12)
;        ld      bc,20
;        call    mem_CopyVRAM
;
;      ld      hl,lmsg3
;        ld      de,_SCRN0+0+(32*13)
;        ld      bc,20
;        call    mem_CopyVRAM

.loop:
        call    pad_Read
        or      a
        jr      z,.loop

        ret

;                12345678901234567890
cmsg1:  db      "   Press A or B.    "


; Calculate Carrier Frequency
; Exit: DE = Pulse length
;       BC = Space length

CalcT:
        di

        ld      a,$c0
        ld      hl,IRRecBuf
        ld      bc,$0256
        ld      [c],a       ; Enable IR receive

        ld      a,[HeaderFlag]
        or      a               ; Is there a header?
        jr      z,.loop4        ; no

; Wait for first IR pulse
.loop:  ld      a,[c]
        and     b
        jr      nz,.loop

; Wait for header pulse to go away
.loop2:
        ld      d,20*3
.loop3:
        dec     d
        jr      z,.loop4

        ld      a,[c]
        and     b
        jr      nz,.loop3

        jr      .loop2

; Wait for second IR pulse
.loop4: ld      a,[c]
        and     b
        jr      nz,.loop4

; Record raw IR data @ 262,144Hz sample rate (3.8uS)

        ld      [hl+],a
        rept    BUFSIZE
        ld      a,[c]
        ld      [hl+],a
        endr

        ei

; Remove 1's from first pulse
; (i.e. remove the carrier and
; make the pulse appear solid.)

        ld      hl,IRRecBuf
.loop10:
        ld      a,[hl+]
        and     2
        jr      z,.loop10

        dec     hl
        ld      d,h
        ld      e,l

        ld      b,20*4
.loop12:
        dec     b
        jr      z,.loop13

        ld      a,[hl+]
        and     2
        jr      nz,.loop12

        ld      h,d
        ld      l,e
        ld      [hl],0

        jr      .loop10

.loop13:

; Find the length of the first pulse

        ld      hl,IRRecBuf
        ld      bc,0
LenFP:
        inc     bc

        ld      a,[hl+]
        and     2
        jr      z,LenFP

        push    bc

; Find the length of the first space

        dec     hl
        ld      bc,0
.loop:
        inc     bc

        ld      a,[hl+]
        and     2
        jr      nz,.loop

        pop     de

        ret

; Display Frequency
;  Entry: B = $31-$34, 32kHz
;         B = $35-$39, 36kHz
;         B = $3a-$3b, 37.916kHz
;         B = $3c,     39.2kHz
;         B = $3d-$3e, 40kHz
;         B = $55-$59, 56.875kHz

DisplayCarrierFreq:
        push    af
        ld      a,b

        cp      $31
        jr      c,DCF1

        ld      hl,Freq32
        cp      $35
        jr      c,DCF2

        ld      hl,Freq36
        cp      $3a
        jr      c,DCF2

        ld      hl,Freq38
        cp      $3c
        jr      c,DCF2

        ld      hl,Freq39
        cp      $3d
        jr      c,DCF2

        ld      hl,Freq40
        cp      $55
        jr      c,DCF2

        ld      hl,Freq57
        cp      $5a
        jr      c,DCF2

; Freq = Unknown
DCF1:
        push    af
        ld      hl,FreqUn
        call    DCF3
        pop     af

        ld      hl,_SCRN0+13+(32*6)
        call    DisplayHexByte
        pop     af
        ret

; Freq = xxxx
DCF2:
        call    DCF3
        pop     af
        ret

DCF3:
        push    hl
        ld      hl,FreqMsg
        ld      de,_SCRN0+3+(32*5)
        ld      bc,14
        call    mem_CopyVRAM
        pop     hl

        ld      de,_SCRN0+5+(32*6)
        ld      bc,9
        call    mem_CopyVRAM

        ret

FreqMsg: db     "Carrier Freq ="
FreqUn: db      "Unknown  "
Freq32: db      "32kHz    "
Freq36: db      "36kHz    "
Freq38: db      "37.916kHz"
Freq39: db      "39.2kHz  "
Freq40: db      "40kHz    "
Freq57: db      "56.875kHz"

; Calculate Carrier Frequency
;  Entry: A = 0, do no header low precision calculation
;         A = 1, do header high precision calculation
;  Exit:  B = $31, 32kHz
;         B = $38, 36kHz
;         B = $3b, 37.916kHz
;         B = $3c, 39.2kHz
;         B = $3d, 40kHz
;         B = $57, 56.875kHz

CalcCarrierFreq:
        or      a               ; Is a header present?
        jr      nz,CCF2         ; yes

        ld      de,100
        call    CalcCFLoop
        ld      a,b
        add     a,a
        add     a,a
        ld      b,a
        ret
CCF2:
        ld      de,400
        call    CalcCFLoop
        ret


CalcCFLoop:
        ld      hl,IRRecBuf
stat2:
        ld      a,[hl+]
        and     2
        jr      z,stat2

        push    de
        ld      de,40
        add     hl,de
        pop     de

        ld      bc,0
stat3:
        ld      a,[hl]
        and     2
        cp      2               ; Is this a high transition?
        jr      nz,stat4        ; no

        cp      c               ; Is this the same as last byte?
        jr      z,stat4         ; yes, ignore

        inc     b               ; Increment number of waveforms
stat4:
        ld      a,[hl]
        and     2
        ld      c,a
        inc     hl

        dec     de
        ld      a,d
        or      e
        jr      nz,stat3

        ret

; Detect header in IR stream
; Exit: A = 1, header present
;       A = 0, no header present

DetectHeader:
        ld      hl,IRRecBuf

; Wait for first pulse to go away

        ld      bc,512
.loop2:

        ld      d,20*3
.loop3:
        dec     d
        jr      z,.loop4

        dec     bc
        ld      a,b
        or      c
        jr      z,.loop5

        ld      a,[hl+]
        and     2
        jr      nz,.loop3

        jr      .loop2

.loop4:
        ld      a,0
        ld      [HeaderFlag],a
        ret
.loop5:
        ld      a,1
        ld      [HeaderFlag],a
        ret

; Obtain data from IR input

GetAllRaw:
        di

; Switch to double speed mode

;        xor     a
;        ld      [rIE],a
;        ld      [rIF],a
;
;        ld      a,$30
;        ld      [rP1],a
;
;        ld      a,1
;        ld      [$ff4d],a
;
;        stop
;        nop

        ld      a,$c0
        ld      hl,IRRecBuf
        ld      bc,$0256
        ld      [c],a       ; Enable IR receive

; Wait for first IR pulse
.loop:  ld      a,[c]
        and     b
        jr      nz,.loop
 if 0
; Wait for first pulse to go away
.loop2:
        ld      d,20*3
.loop3:
        dec     d
        jr      z,.loop4

        ld      a,[c]
        and     b
        jr      nz,.loop3

        jr      .loop2

; Wait for second IR pulse
.loop4: ld      a,[c]
        and     b
        jr      nz,.loop4
 endc

; Record raw IR data @ 262,144Hz sample rate (3.8147uS)

        ld      [hl+],a
        rept    BUFSIZE
        ld      a,[c]
        ld      [hl+],a
        endr

        ei
        ret

DisplayHexByte:
        push    af
        swap    a
        call    DisplayNibble
        pop     af
        call    DisplayNibble
        ret

DisplayNibble:
        push    bc
        and     $f
        add     $30
        cp      $3a
        jr      c,DispN2
        add     "a"-"0"-10
DispN2:
        ld      b,a
.wait
        ld      a,[$ff41]
        and     2
        jr      nz,.wait

        ld      [hl],b
        inc     hl
        pop     bc
        ret


IRDelayOff:
        ld      bc,10000
.loop2:
        rept 16
        nop
        endr

        dec     bc
        ld      a,b
        or      c
        jr      nz,.loop2

        ret

DrawButtons:
        ld      b,a
        ld      hl,AddrList
        ld      d,8
.loop:
        rlc     b
        ld      a,b
        and     1

        call    DrawButton

        dec     d
        jr      nz,.loop
        ret

DrawButton:
        push    bc
        push    de
        ld      e,[hl]
        inc     hl
        ld      d,[hl]
        inc     hl
        push    hl

        ld      b,32
        or      a
        jr      z,.wait
        ld      b,0
.wait:
        ldh     a,[$41]
        and     2
        jr      nz,.wait

        ld      a,b
        ld      [de],a

        pop     hl
        pop     de
        pop     bc
        ret

AddrList:
        dw      $9800+6+(32*10)  ;down
        dw      $9800+6+(32*8)   ;up
        dw      $9800+5+(32*9)   ;left
        dw      $9800+7+(32*9)   ;right
        dw      $9800+12+(32*13) ;start
        dw      $9800+9+(32*13)  ;select
        dw      $9800+14+(32*9)  ;B
        dw      $9800+17+(32*8)  ;A

;
; convert integer to string
; de = addr of string
; hl = value to convert
; exit:	de = updated value
;
bta:
	xor	a	     ;set for no leading zeroes
	ld	bc,-10000
	call	rsub
	ld	bc,-1000
	call	rsub
	ld	bc,-100
	call	rsub
	ld	bc,-10
	call	rsub
	ld	bc,-1
	call	rsub
	ret	nz

        di
.loop:  ld      a,[$ff41]
        and     2
        jr      nz,.loop

        ld      a,"0"
        ld      [de],a
        ei

	inc	de
	ret
;
; Take value in hl sub # in bc the
; most possible times.
; Put value on string at de.
; If a=0 then don't put zero on string.
; Return non-zero if a put on string
;
rsub:	push	de
	ld	d,-1
        di                      ;<----+
rsub1:  push    hl              ;     |
        inc     sp              ;     |
        inc     sp              ;     |
        inc     d               ;     |
        add     hl,bc           ;     +---- Kill interrupts since we're
        jr      c,rsub1         ;     |     unusually messing with stack.
                                ;     |
        dec     sp              ;     |
        dec     sp              ;     |
        ei                      ;<----+
	pop	hl
	ld	b,d
	pop	de
	or	b	     ;a gets 0 if a was 0 and b is 0
	ret	z

        ld      a,"0"
        add     a,b
        push    af

        di
.loop:  ld      a,[$ff41]
        and     2
        jr      nz,.loop

        pop     af
        ld      [de],a
        ei

	inc	de
	ret

;*
;*  QMUL  -  Quick 16 x 16 Unsigned Multiply
;*
;*
;*         By:           Jon Tara
;*                       MicroControl Software
;*                       1300 E. Lafayette #2808
;*                       Detroit, Michigan  48207
;*                       (313) 393-2916
;*
;*        This software is placed in the public domain.
;*
;*
;*  On entry:  DE = Multiplicand
;*             HL = Multiplier
;*
;*  On exit:   BCDE = Product
;*
;*  AF destroyed
;*
;*  Revision 1.00  01/21/82   JT  Initial Release
;*  Revision 1.10  25/Apr/97  Bug fixes - Jeff Frohwein
;*

mul_DE_HL_BCDEu::
        push    hl              ; Save multiplier.
        ld      c,h             ; Save MSBs of multiplier.
        ld      a,l             ; LSBs to A for an 8 x 16 multiply.

        ld      b,0             ; Handy 0 to B for carry propagation.
        ld      h,b             ; Init LSBs of product.
        ld      l,b

        add     a,a             ; Test multiplier bit.
        jr      nc,.mul2
        add     hl,de           ; Add multiplicand to product.
        adc     a,b             ; (Product in AHL)
.mul2   add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul3
        add     hl,de           ; Add multiplicand to product.
        adc     a,b             ; (Product in AHL)
.mul3   add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul4
        add     hl,de           ; Add multiplicand to product.
        adc     a,b             ; (Product in AHL)
.mul4   add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul5
        add     hl,de           ; Add multiplicand to product.
        adc     a,b             ; (Product in AHL)
.mul5   add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul6
        add     hl,de           ; Add multiplicand to product.
        adc     a,b             ; (Product in AHL)
.mul6   add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul7
        add     hl,de           ; Add multiplicand to product.
        adc     a,b             ; (Product in AHL)
.mul7   add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul8
        add     hl,de           ; Add multiplicand to product.
        adc     a,b             ; (Product in AHL)
.mul8   add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul9
        add     hl,de           ; Add multiplicand to product.
        adc     a,b             ; (Product in AHL)

.mul9   push    hl              ; Save LSBs in stack.
        ld      h,b             ; Zero second product.
        ld      l,b             ; .
        ld      b,a             ; Save MSBs of first product in B
        ld      a,c             ; Get MSBs of multiplier.
        ld      c,h             ; Handy 0 in C this time.

        add     a,a             ; Test multiplier bit.
        jr      nc,.mul20
        add     hl,de           ; Add multiplicand to product.
        adc     a,c             ; (Product in AHL)
.mul20  add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul21
        add     hl,de           ; Add multiplicand to product.
        adc     a,c             ; (Product in AHL)
.mul21  add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul22
        add     hl,de           ; Add multiplicand to product.
        adc     a,c             ; (Product in AHL)
.mul22  add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul23
        add     hl,de           ; Add multiplicand to product.
        adc     a,c             ; (Product in AHL)
.mul23  add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul24
        add     hl,de           ; Add multiplicand to product.
        adc     a,c             ; (Product in AHL)
.mul24  add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul25
        add     hl,de           ; Add multiplicand to product.
        adc     a,c             ; (Product in AHL)
.mul25  add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul26
        add     hl,de           ; Add multiplicand to product.
        adc     a,c             ; (Product in AHL)
.mul26  add     hl,hl           ; Shift product left.
        adc     a,a             ; Test multiplier bit.
        jr      nc,.mul27
        add     hl,de           ; Add multiplicand to product.
        adc     a,c             ; (Product in AHL)

.mul27  pop     de              ; Fetch LSBs of 1st product.
        ld      c,a             ; Add partial products.
        ld      a,d             ; .
        add     a,l             ; .
        ld      d,a             ; .
        ld      a,b             ; .
        adc     a,h             ; .
        ld      h,a             ; .
        ld      a,c             ; .
        adc     a,0             ; .
        ld      b,a             ; .
        ld      c,h             ; .
        pop     hl              ; Restore multiplier.
        ret

; 16 bit division
; DE = DE / BC, BC = remainder

div1616:
        ld      hl,temp
        ld      [hl],c
        inc     hl
        ld      [hl],b
        inc     hl
        ld      [hl],17
        ld      bc,0
nxtbit: ld      hl,count
        ld      a,e
        rla
        ld      e,a
        ld      a,d
        rla
        ld      d,a
        dec     [hl]
        ret     z
        ld      a,c
        rla
        ld      c,a
        ld      a,b
        rla
        ld      b,a
        dec     hl
        dec     hl
        ld      a,c
        sub     [hl]
        ld      c,a
        inc     hl
        ld      a,b
        sbc     a,[hl]
        ld      b,a
        jr      nc,noadd

        dec     hl
        ld      a,c
        add     a,[hl]
        ld      c,a
        inc     hl
        ld      a,b
        adc     a,[hl]
        ld      b,a
noadd:  ccf
        jr      nxtbit


; Wait for VBlank to occur

WaitForVBlank:
        ld      a,[VBlankDone]
        or      a
        jr      z,WaitForVBlank

        xor     a
        ld      [VBlankDone],a
        ret

; Init video stuff

Initialize:
        call    ScreenOff

        ld      hl,VBlankFirst
        ld      de,SpriteDMA
        ld      bc,VBlankLast-VBlankFirst
        call    mem_Copy

        ld      hl,SpriteTab
        ld      bc,4*40
        xor     a
        call    mem_Set                 ; Clear sprite table

        ld      hl,$fe00
        ld      bc,4*40
        xor     a
        call    mem_Set                 ; Clear sprite mem

        ld      a,16+4
        ld      [SpriteTab+0],a
        ld      a,8
        ld      [SpriteTab+1],a
        ld      a,1
        ld      [SpriteTab+2],a
        ld      a,0
        ld      [SpriteTab+3],a         ; Put a sprite in upper left corner

        xor     a
        ldh     [rIE],a

        xor     a
        ldh     [rSCX],a
        ldh     [rSCY],a

        ld      a,$e4
        ldh     [rBGP],a
        ldh     [rOBP0],a
        ldh     [rOBP1],a

        ld      d,0
        call    ClearCGBAttrMap

        ld      hl,Select_CGBPalettes
        call    SetCGB_OBJP
        ld      hl,Select_CGBPalettes
        call    SetCGB_BGP

        ld      hl,CharSet
        ld      de,_VRAM
        ld      bc,256*8
        call    mem_CopyMono            ; Load font

        ld      hl,_SCRN0
        ld      bc,32*32
        ld      a,32
        call    mem_Set                 ; Clear screen
        ret

; *** Turn screen on ***

ScreenOn:
        ld      a,LCDCF_ON|LCDCF_BG8000|LCDCF_BG9800|LCDCF_BGON|LCDCF_OBJ8|LCDCF_OBJON
        ldh     [rLCDC],a
        ret

; Turn off the LCD

ScreenOff:
        ldh     a,[rLCDC]
        bit     7,a             ; Is LCD already off?
        ret     z               ; yes, exit

        ldh     a,[rIE]
        push    af
        res     0,a
        ldh     [rIE],a

.wait:  ldh     a,[rLY]
        cp      145
        jr      nz,.wait

        ldh     a,[rLCDC]
	res	7,a
        ldh     [rLCDC],a

        pop     af
        ldh     [rIE],a
        ret

; *** Display screen text message ***

DisplayScreenMsgs:
        ld      hl,RemoteMsg
        ld      de,_SCRN0+5+(32*3)
        ld      bc,13
        call    mem_Copy

        call    DrawOutlines

        ld      hl,ChanUp
        ld      de,_SCRN0+5+(32*6)
        ld      bc,3
        call    mem_Copy

        ld      hl,ChanDown
        ld      de,_SCRN0+5+(32*12)
        ld      bc,3
        call    mem_Copy

        ld      hl,VolUp
        ld      de,_SCRN0+9+(32*9)
        ld      bc,4
        call    mem_Copy

        ld      hl,VolDown
        ld      de,_SCRN0+0+(32*9)
        ld      bc,4
        call    mem_Copy

        ld      hl,Power
        ld      de,_SCRN0+6+(32*15)
        ld      bc,10
        call    mem_Copy

        ret

; *** Draw outline of virtual buttons ***

DrawOutlines:
        ld      hl,Outlines
        ld      de,_SCRN0+(32*2)
        ld      bc,13*32
        call    mem_CopyVRAM

        ret

RemoteMsg:
        DB      "Remote Control"

VolUp:
        DB      "Vol+"
VolDown:
        DB      "Vol-"
ChanUp:
        DB      "Ch+"
ChanDown:
        DB      "Ch-"
Power:
        DB      "Mute Power"

; *** Virtual buttons map ***

Outlines:
        DB      " ",$c9,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$bb," xxxxxxxxxxxx"
        DB      " ",$ba," Remote Control ",$ba," xxxxxxxxxxxx"
        DB      " ",$c8,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$cd,$bc," xxxxxxxxxxxx"
        DB      "                    xxxxxxxxxxxx"
        DB      "                    xxxxxxxxxxxx"
        DB      "     ",$c9,$cd,$bb,"        ",$c9,$cd,$bb," xxxxxxxxxxxx"
        DB      "    ",$c9,$bc,32,$c8,$bb,"    ",$c9,$cd,$bb,$ba,32,$ba," xxxxxxxxxxxx"
        DB      "    ",$ba,"   ",$ba,"    ",$ba,32,$ba,$c8,$cd,$bc," xxxxxxxxxxxx"
        DB      "    ",$c8,$bb,32,$c9,$bc,"    ",$c8,$cd,$bc,"    xxxxxxxxxxxx"
        DB      "     ",$c8,$cd,$bc,"            xxxxxxxxxxxx"
        DB      "        ",$c9,$cd,$bb,$c9,$cd,$bb,"      xxxxxxxxxxxx"
        DB      "        ",$ba,$20,$ba,$ba,$20,$ba,"      xxxxxxxxxxxx"
        DB      "        ",$c8,$cd,$bc,$c8,$cd,$bc,"      xxxxxxxxxxxx"

; Set Color GB Attribute memory to value in D

ClearCGBAttrMap:
        ld      hl,$9800

        ld      a,1
        ld      [$ff4f],a

        ld      b,32*2          ; 32*32
.memc2:
        ld      a,d
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a
        ld      [hl+],a

        dec     b
        jr      nz,.memc2

        xor     a
        ld      [$ff4f],a
        ret

; Set background palettes
SetCGB_BGP:
        ld      a,$80
        ldh     [$68],a

        ld      bc,$4069        ; b = 64, c = $69
.loop:
        ld      a,[hl+]
        ld      [c],a
        dec     b
        jr      nz,.loop

        ret

; Set background palettes
SetCGB_OBJP:

        ld      a,$80
        ldh     [$6a],a

        ld      bc,$406b        ; b = 64, c = $69
.loop:
        ld      a,[hl+]
        ld      [c],a
        dec     b
        jr      nz,.loop

        ret

Select_CGBPalettes:
        RGBSet  255,255,255     ; White         Background palette
        RGBSet  191,191,191     ; Light Grey
        RGBSet  127,127,127     ; Dark Grey
        RGBSet  0,0,0           ; Black

;        RGBSet  0,0,0           ; Black
;        RGBSet  127,127,127     ; Dark Grey
;        RGBSet  191,191,191     ; Light Grey
;        RGBSet  255,255,255     ; White         Background palette

        RGBSet  254,188,241
        RGBSet  254,97,235
        RGBSet  166,0,157
        RGBSet  0,0,0

        RGBSet  168,255,255
        RGBSet  0,194,200
        RGBSet  1,136,139
        RGBSet  1,48,54

        RGBSet  255,255,255
        RGBSet  14,230,241
        RGBSet  102,98,64
        RGBSet  40,30,21

        RGBSet  255,255,255
        RGBSet  174,166,102
        RGBSet  0,109,219
        RGBSet  0,0,0

        RGBSet  203,184,160
        RGBSet  141,120,82
        RGBSet  170,0,85
        RGBSet  43,19,19

        RGBSet  255,255,255
        RGBSet  169,167,86
        RGBSet  0,128,128
        RGBSet  0,64,0

        RGBSet  255,255,0
        RGBSet  253,150,94
        RGBSet  219,0,0
        RGBSet  66,34,34

CharSet:
        chr_IBMPC1      1,8

